@@ -1,7 +1,7 @@
 # Deb-o-Matic
 #
 # Copyright (C) 2008-2009 David Futcher
-# Copyright (C) 2012-2022 Luca Falavigna
+# Copyright (C) 2012-2025 Luca Falavigna
 #
 # Author: David Futcher <bobbo@ubuntu.com>
 #
@@ -22,9 +22,8 @@
 import os
 from glob import glob
 from logging import debug
-from re import findall
 from sys import path
-from toposort import toposort_flatten as toposort
+from graphlib import TopologicalSorter as TS, CycleError
 
 from Debomatic import dom
 from .process import ModulePool
@@ -199,9 +198,10 @@ class Module():
                 _afters = self._instances[instance]._after
                 modules[instance] = _deps.union(_afters)
         try:
-            return [m for m in toposort(modules) if m in self._instances]
-        except ValueError as e:
-            circular = findall(r'.*?\(\'?(\S+?)\'?,', e.args[0])
+            return [m for m in TS(modules).static_order()
+                    if m in self._instances]
+        except CycleError as e:
+            circular = set(e.args[1])
             for instance in circular:
                 self._instances[instance]._disabled = True
             debug(_('Circular dependencies found, disabled modules: %s')
