@@ -1,7 +1,7 @@
 # Deb-o-Matic - AutoPkgTest module
 #
 # Copyright (C) 2014 Leo Iannacone
-# Copyright (C) 2015-2022 Luca Falavigna
+# Copyright (C) 2015-2025 Luca Falavigna
 #
 # Authors: Leo Iannacone <l3on@ubuntu.com>
 #
@@ -117,10 +117,11 @@ class DebomaticModule_AutoPkgTest:
                         output.write('\n\n')
                 output.flush()
 
+        chrootname = f'{args.distribution}-{args.architecture}-debomatic'
         adt = [self.adt, '--apt-upgrade', '--output-dir', self.resultdir_adt,
                '--summary', os.path.join(self.resultdir_adt, self.summary),
-               self.changesfile, '--', 'schroot',
-               '%s-%s-debomatic' % (args.distribution, args.architecture)]
+               self.changesfile, '--', 'unshare', '-t',
+               os.path.expanduser(f'~/.cache/sbuild/{chrootname}.tar.zst')]
         if self.options:
             adt.insert(-4, self.options)
 
